/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;
import java.net.*;
import java.awt.image.*;

/**
 * A class that produces a ImageCheckButton. It is similar to ImageButton.
 * It has two states like checkbox.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class ImageCheckButton extends ImageButton {

	/** When ImageCheckButton is swith up, parent will get the event with this id. */
	public static final int IMG_BUTTON_UP = 8556152;

	/** When ImageCheckButton is swith down, parent will get the event with this id. */
	public static final int IMG_BUTTON_DOWN = 8554010;
	
	/** Determine whether button can be switched up using mouse. If it is set to false,
	 *  button can be switched up only by call setState
	 * @see #setUpMouseMode
	 * @see #getUpMouseMode
	 */
	private boolean upMouseMode = true;
	
	/** Create an ImageCheckButton with the default image.
	 * @see ImageLabel#defaultImageString
	 */
	public ImageCheckButton() {
		super();
	}
	
	/** Create an ImageCheckButton using the image at URL specified by the string.
	 * @param imageURLString A String specifying the URL of the image.
	 */
	public ImageCheckButton(String imageURLString) {
		super(imageURLString);
	}
	
	/** Create an ImageCheckButton using the image at URL specified.
	 * @param imageURL The URL of the image.
	 */
	public ImageCheckButton(URL imageURL) {
		super(imageURL);
	}
	
	/** Create an ImageCheckButton using the image specified. You would only want to
	 *  call it directly if you already have an image (e.g. created via
	 *  createImage).
	 * @param image The image.
	 */
	public ImageCheckButton(Image image) {
		super(image);
	}
	
	/** Sets new value of upMouseMode.
	 * @see #upMouseMode
	 * @see #getUpMouseMode
	 */
	public void setUpMouseMode(boolean mode) {
		upMouseMode = mode;
	}
	
	/** Gets actual value of upMouseMode.
	 * @see #upMouseMode
	 * @see #setUpMouseMode
	 */
	public boolean getUpMouseMode() {
		return upMouseMode;
	}
	
	public boolean mouseDown(Event event, int x, int y) {
		mouseIsDown = true;
		if (isDown) {
			if (upMouseMode) {
				isDown = false;
				event.id = IMG_BUTTON_UP;
			}
			else {
				mouseIsDown = false;
				repaint();
				return false;
			}
		}
		else {
			isDown = true;
			event.id = IMG_BUTTON_DOWN;
		}
		event.arg = this;
		repaint();
		return false;
	}

	public boolean mouseUp(Event event, int x, int y) {
		mouseIsDown = false;
		return false;
	}

	public boolean mouseExit(Event event, int x, int y) {
		return false;
	}

	/** Sets new state of ImageCheckButton.
	 * @param aState New state - boolean value.
	 * @see ImageButton#isDown
	 * @see #getState
	 */
	public void setState(boolean aState) {
		if (isDown != aState) {
			isDown = aState;
			repaint();
		}
	}

	/** Sets new state of ImageCheckButton.
	 * @param aState New state - constants IMG_BUTTON_UP or IMG_BUTTON_DOWN
	 * @see ImageButton#isDown
	 * @see #IMG_BUTTON_UP
	 * @see #IMG_BUTTON_DOWN
	 * @see #getState
	 */
	public void setState(int aState) {
		if (aState == IMG_BUTTON_UP)
			setState(false);
		if (aState == IMG_BUTTON_DOWN)
			setState(true);
	}
	
	/** Gets current state of ImageCheckButton.
	 * @return Current state.
	 * @see ImageButton#isDown
	 */
	public boolean getState() {
		return isDown;
	}
}
