/**
* Xelfi
*/

package xelfi.awt;

import java.awt.Component;
import java.awt.Event;

/**
 * Class Timer implements behaviour of "timer". It periodically sends "ACTION" message to its 
 * owner (with string "TimerEvent" in arg).
 * The timer can be started/stopped, it is possible to set delay time and two modes of
 * operation are possible - cycle-mode, where the timer keeps sending messages again and again,
 * and single-mode, where the message is only sent once after specified delay time after
 * the timer is started.
 * The owner of the timer must be descendant of AWT Component class.
 *
 * @version 	1.03, 25 Jan 1997 
 * @author 		Ian Formanek
 */

public class Timer {
	TimerThread tt;

	public Timer(Component parent, int ms) {
		this(parent, ms, true); 
	}

	public Timer(Component parent, int ms, boolean cycle) {
		tt = new TimerThread(parent, ms, cycle); 
	}

	public void start() {
		tt.timer = this;
		tt.start();
	}

	public void stop() {
//		tt.stop();
		tt.suspend();
	}

	public void setDelay(int aMs) {
		tt.ms = aMs;
	}

	public void setCycle(boolean aCycle) {
		tt.cycle = aCycle;
	}
	
}

class TimerThread extends Thread {
	Timer timer;
	Component parent;
	int ms;
	boolean cycle;

	public TimerThread(Component aParent, int aMs, boolean aCycle) {
		parent = aParent;
		ms = aMs;
		cycle = aCycle;
	}

	public void run() {
		Event timerEvent = new Event(timer, Event.ACTION_EVENT,"TimerEvent"); 
		while (true) {
			try {
				sleep(ms);
			} catch (InterruptedException e) {}
		parent.postEvent(timerEvent);
		if (!cycle) break;
		}
	}
}