/**
* Xelfi programming environment
*/

package xelfi.browser.document;

import java.util.*;
import java.io.*;
import java.awt.*;
import xelfi.browser.util.Member;

/**
 * From this class all Browser components are extended.
 * It's only use in multiple selectioned Browser windows
 * as detach line.
 *
 * @version 1.0
 * @author Jan Jancura
 */
public class BrComponent implements Member
{

   /**
    * This parameters stores name of the component.
    */
    public String name;

   /**
    * This method returns name for sorting and etc.
    * @return Name.
    */
    public String getKey()
    {
        return name;
    }

   /**
    * Constructor.
    */
    public BrComponent()
    {}

   /**
    * This constructor is used in detach line.
    *
    * @param pName The name of the parent of the following
    *  components.
    */
    public BrComponent(String pName)
    {
        char[] pom = new char[Math.max(26-pName.length(),0)];
        int i,k = pom.length;
        for(i=0;i<k;i++)
            pom[i] = '-';
        name = "--" + pName + pom;
    }

   /**
     * This method return name with modifiers and other
     * parameters.
     * @param bs Actual settings.
     * @param depth If this component listbox is hierarched
     * this parameter stores information about components depth
     * in this tree.
     * @return Full name
     */
    public String getFullName(BrSettings bs,int depth)
    {
        return name;
    }

   /**
    * This method returns actually visible subcomponents of this
    * component. This components are stored to the given array
    * form @see from index.
    *
    * @param bs Actual settings.
    * @param c The array of the components.
    * @param from First non occupied position.
    * @return The index of the first non occupied position
    *  past adding components to the array.
    */
    public int getComponents(BrSettings bs,BrComponent[] c,int from)
       throws FileNotFoundException,IOException
    {
        return from;
    }

   /**
    * This method returns a number of actually visible
    * subcomponents.
    *
    * @param bs Actuall settings
    * @return Number of visible subcomponents.
    */
    public int getComponentsNumber(BrSettings bp)
       throws FileNotFoundException,IOException
    {
        return 0;
    }

   /**
    * This method allows adding subcomponents.
    *
    * @param c Component to add.
    * @return True if operation wos succesfull.
    */
    public boolean addComponent(BrComponent c)
    {
        return false;
    }

   /**
     * This method returns true, if selecting of this component
     * is allowed.
     * @return false
     */
    public boolean selectable()
    {
        return false;
    }

   /**
     * This method opens view for this component.
     * @param f Parent frame
     */
    public void openView(Frame f)
    {}

   /**
    * This method returns string representation of this class.
    */
    public String toString()
    {
        return name;
    }

    protected static String strToHint(String str,int indent)
    {
        StringTokenizer st = new StringTokenizer(str,"\n\r");
        StringBuffer b = new StringBuffer();
        int i;
        while( st.hasMoreTokens() )
        {
            for(i=0;i<indent;i++) b.append(' ');
            b.append(" * ").append(st.nextToken()).append('\n');
        }
        for(i=0;i<indent;i++) b.append(' ');
        b.append(" */");
        return new String(b);
    }
    static protected String hintToString(String hint)
    {
        StringTokenizer st = new StringTokenizer(hint,"\n\r");
        StringBuffer b = new StringBuffer();
        String s;
        while( st.hasMoreTokens() )
        {
            if( (s = st.nextToken().trim()).length() > 0 )
            {
                if( s.startsWith("*") )
                {
                    //if( s.length() == 1 ) continue;
                    s = s.substring(1).trim();
                }
                b.append(s).append('\n');
            }
        }
        return (s = new String(b)).substring(0,java.lang.Math.max
            (s.length() - 2,0));
    }
}












