/** XelfiCommpilerOutput class
* @author: Filip Dvorak
* @version: 1.2
*/

package xelfi.compiler;

import java.io.*;
import java.util.*;

import xelfi.top.*;
import xelfi.browser.document.XFile;
import xelfi.util.*;
import xelfi.editor.XelfiFile;

  /** Catches errors from compilation, assumes that ordinar error format is
   <file>:<line number>:<message>\n
   <source text involved in the error>\n
   <error pointer '^'>\n
  */
class XelfiCompilerOutput extends OutputStream
{
    /** Keeps not finished last row of text - items are characters */
  protected StringBuffer buff;
    /** Status of pushing lines of text into List */
  protected int status;

    /** valid only in ERROR_LINE a ERROR_PTR mode */
  protected ErrorDescription currentError;

    /** Status of reading a text from write methods */
  protected static final int SCAN_LINE = 0;
  protected static final int ERROR_LINE = 1;
  protected static final int ERROR_PTR = 2;

  protected Hashtable errorHistory;

  protected Output outWin;
   
    // for caching last resolved filename to class name
  private String lastResolvedPath, lastResolvedClassName;

  XelfiCompilerOutput()
  {
    buff = new StringBuffer(200);
      // connect to a output window
   outWin = TopLevel.getOutput();
   errorHistory = new Hashtable(50, 0.82f);
   resetStatus();
  }

  boolean errorHistoryContains(String clName)
  {
    return errorHistory.get(clName) != null;
  }

    /** Utility used internally. path is name for a class source generated by compiler. Returns class name or null
      if no such class.
      Supposing tha filenames from compiler are preceded by path from classpath system property.
      NOTE: The same utility provides XelfiFile now, so this method may be not called .
     */
  private String resolveClassName(String path)
  {
    if(path.equals(lastResolvedPath)) return lastResolvedClassName;
      // get matching classpath
    //Enumeration classpath = PathScanner.elements(System.getProperty("java.class.path"));
    Enumeration classpath = XFile.getAllClassPathsDir();
    while(classpath.hasMoreElements())
    {
      String match = (String)classpath.nextElement();
      if(path.regionMatches(false, 0, match, 0, match.length()))
      {
          // change a/b/C.java --> a.b.C
        String name = path.substring(match.length()), tmp = null;
        StringTokenizer name_token = new StringTokenizer(name, File.separator, false);
        StringBuffer name_class = new StringBuffer(name.length());
        while(name_token.hasMoreElements())
        {
          tmp = (String)name_token.nextElement();
          if(!name_token.hasMoreElements()) break;  // last element is name of class
          name_class.append(tmp);
          name_class.append(".");
        }
          // name of the class is in tmp
        if(tmp.endsWith(".java"))
        {
          name_class.append(tmp.substring(0, tmp.length() - 5));
          lastResolvedPath = path;
          lastResolvedClassName = name_class.toString();
          return lastResolvedClassName;
        }
        else return null;
      }
    }
    // no classpath element match
    return null;
  }

  public void write(byte b[])
  {
    write(b, 0, b.length);
  }

  public void write(byte b[], int off, int len)
  {
    String s = new String(b, 0, off, len), c = null;
    StringTokenizer t = new StringTokenizer(s, "\n", true);

    if(t.countTokens() > 1)
    {    // there is a newline character in s
      String bufik = buff.toString();
      buff = new StringBuffer(200);
        // first line
      while(!(c = t.nextToken()).equals("\n"))
        bufik = bufik.concat(c);
      produceLine(bufik);

      bufik = "";
      while(t.hasMoreTokens())  // resolve the other lines of text
        if((c = t.nextToken()).equals("\n"))
        {
          produceLine(bufik);
          bufik = "";
        }
        else bufik = c;

      buff.append(bufik);
    }
    else buff.append(s);
  }

  public void write(int b)
  {
    char c = (char)b;
    if(c == '\n')
    {
      produceLine(buff.toString());
      buff = new StringBuffer(200);
    }
    else buff.append(c);
  }

    /** Output line of the text generated by write() functions */
  protected void produceLine(String l)
  {
    int lineId;

    switch(status)
    {
      case SCAN_LINE:
        try
        {
          StringTokenizer t = new StringTokenizer(l, ":");

            // find the filename
          StringBuffer fname = new StringBuffer(128);
          fname.append(t.nextToken());
          String s2 = fname.toString(), s1 = null, tmp = null;
          File f2 = new File(s2), f1 = null;
          do
          {
            s1 = s2;
            f1 = f2;
            fname.append(":");
            fname.append(tmp = t.nextToken());
            f2 = new File(s2 = fname.toString());
          } while(!(f1.isFile() && !f2.exists()));
            // f1 is the filename and tmp contains next token

            // find class name
          String className = null;
          // case: using my own resolveClassName:
          if((className = resolveClassName(f1.toString())) == null) throw new ErrorFormatException();

            // case: using XelfiFile
          //if((className = XelfiFile.find(f1).getPackageAndName()) == null) throw new ErrorFormatException();

//          System.out.println("Resolved: " + className);

            // get line number
          int lineNum;
          String i = tmp;  // next token
          lineNum = Integer.parseInt(i);

          currentError = new ErrorDescription(lineNum);
          ErrorDatabase.addError(currentError, className);

          StringBuffer msg = new StringBuffer(128);
          msg.append(className);
          msg.append("(");
          msg.append(lineNum);
          msg.append("): ");
            // append rest of the tokenized line
          StringBuffer report = new StringBuffer(128);
          while(t.hasMoreTokens())
          {
            report.append(t.nextToken());
            if(t.hasMoreElements()) report.append(":");
          }
          currentError.setOutputLine(outWin.outLine(msg.toString() + (tmp = report.toString()), currentError));
          currentError.setComment(tmp);
          errorHistory.put(className, new Boolean(true));  // need some Object tu put into hashtable
          status = ERROR_LINE;
        }
        catch(NumberFormatException e)
        {
          resetStatus();
          outWin.println(l);
        }
        catch(NoSuchElementException e)    // tokenizer exception
        {
          resetStatus();
          outWin.println(l);
        }
/*        catch(IOException e)  // case: using XelfiFile
        {
          resetStatus();
          outWin.println(l);
        }*/
        catch(ErrorFormatException e)
        {
          resetStatus();
          outWin.println(l);
        }
        break;
      case ERROR_LINE:
        status = ERROR_PTR;
        break;
      case ERROR_PTR:
        int col = 0;
        while(l.charAt(col++) == ' ');
        currentError.setColumn(col);
        status = SCAN_LINE;
        break;
    }
  }

  private void resetStatus()
  {
    status = SCAN_LINE;
    currentError = null;
  }

    /** It resets status and outputs the string */
  void topLevelError(String s)
  {
    resetStatus();
      // Flush my buffer
    String tmp = buff.toString();
    if(tmp.length() > 0)
    {
      produceLine(tmp);
      buff = new StringBuffer(200);
    }

    String a = null, b = "";
    StringTokenizer t = new StringTokenizer(s, "\n", true);
    while(t.hasMoreTokens())
      if((a = t.nextToken()).equals("\n"))
      {
//        System.out.println("addig text: \"" + b + "\" pod cislem UNKNOWN");
        outWin.println(b);
        b = "";
      }
      else b = a;
    if(b.length() > 0)  outWin.println(b);
//      System.out.println("addig text: \"" + b + "\" pod cislem UNKNOWN"); // do listu hod b
  }

    /** It resets status and outputs the string */
  void topLevelOutput(String msg)
  {
    resetStatus();
      // Flush my buffer
    String tmp = buff.toString();
    if(tmp.length() > 0)
    {
      produceLine(tmp);
      buff = new StringBuffer(200);
    }

    String a = null, b = "";
    StringTokenizer t = new StringTokenizer(msg, "\n", true);
    while(t.hasMoreTokens())
      if((a = t.nextToken()).equals("\n"))
      {
        outWin.println(b);
        b = "";
      }
      else b = a;
    if(b.length() > 0) outWin.println(b);
  }
}

