/**
 * CallStackView
 *
 * @author Roman Blazevic
 * @version 970208
 */

package xelfi.debugger;

import java.awt.*;
import xelfi.top.TopLevel;
import xelfi.top.TopDialog;

public class CallStackView extends TopDialog
{
	private Debugger document;
	private List list;
	private RemoteStackFrame[] csf = null; // used for call stack choosing

	static final int KEY_ENTER = 10;

	static final int HINT_COMPLETE_UPDATE = 0;

	/**
	 * CallStackView constructor.
	 *
	 * debugger - the document for this view
	 */
	CallStackView(Debugger debugger)
	{
		super(TopLevel.getMenu(), false);
		setTitle("Call Stack");
		//super("Call Stack");
		setBackground(Color.lightGray);

		document = debugger;
		list = new List();
		add("Center", list);
	}

	/**
	 * My handleEvent() implementation.
	 */
	public boolean handleEvent(Event event)
	{
		// items with index csf.length-i-2 contains info about stack frame with index i (variable csf[i])
		int selectedIndex = list.getSelectedIndex();

		if ((event.id == Event.KEY_PRESS && event.key == KEY_ENTER)
			|| (event.id == Event.ACTION_EVENT))
		{
			if (selectedIndex != -1)
			{
				int stackFrameIndex = csf.length-selectedIndex-2;

				document.stackFrameChosen(csf[stackFrameIndex]);
			}
			return true;
		}

		return super.handleEvent(event);
	}

	/**
	 * This method is called when the content of the view must be
	 * updated.
	 */
	void update(int hint, int index)
	{
		if (!isVisible())
			return;

		if (hint != HINT_COMPLETE_UPDATE)
		{
			System.out.println("unsupported hint");
			return;
		}

		csf = document.getCurrentStackFrames();

		if (csf == null)
		{
			list.clear();
			return;
		}

		int indexOfSelected = list.getSelectedIndex();
		if (indexOfSelected != -1)
			list.deselect(indexOfSelected);

		int currentStackFrameIndex = document.getCurrentStackFrameIndex();
		int numberOfLines = csf.length-currentStackFrameIndex-1; // without "system" frame
		// int numberOfLines = csf.length-currentStackFrameIndex;

		if (numberOfLines < list.countItems())
			list.delItems(numberOfLines, list.countItems()-1);

		for(int i = csf.length-2; i >= currentStackFrameIndex; i--) // without "system" frame
	{
			// for(int i = csf.length-1; i >= currentStackFrameIndex; i--)
			// "system" frame is that one with index csf.length-1

			try
			{
        		RemoteStackVariable lv[] = csf[i].getLocalVariables();
				String string = new String();
				int lastParameterIndex = -1;

				for(int j = lv.length-1; j >= 0; j--)
					if (lv[j].methodArgument())
					{
						lastParameterIndex = j; // the last method argument is here
						break;
					}

				for(int j = 0; j <= lastParameterIndex; j++)
					if (lv[j].methodArgument())
					{
						String name = lv[j].getName();

						string += name+": "+document.getCurrentValue(name /*, i*/);
						if (j != lastParameterIndex)
							string += ", ";
					}

				// put one call stack line into the 'string' variable
				string = " ["+(csf.length-i-1)+"] "+csf[i].getRemoteClass().getName()+"."
					+csf[i].getMethodName()+"("+string+")";

				if (csf.length-i-1 > list.countItems())
					list.addItem(string);
				else
					if (list.getItem(csf.length-i-2).compareTo(string) != 0)
						list.replaceItem(string, csf.length-i-2);
			}
			catch(Exception e)
			{
				System.out.println(e);
			}
		}
	}

	/**
	 * My show() implementation.
	 */
	public void show()
	{
		super.show();
		list.requestFocus();
		document.callStackViewShown();
	}
}
