/**
* Xelfi
*/

package xelfi.design.forms.layouts;

import java.io.*;
import java.awt.BorderLayout;
import java.awt.Frame;
import java.awt.Container;
import java.awt.Component;
import java.util.Hashtable;
import java.util.Enumeration;
import xelfi.design.components.*;
import xelfi.design.util.XFDesignConstants;

/* 
 * Class XBorderLayout 
 *
 * @version 	1.20, 15 Mar 1997 (From 1.10 extends BorderLayout)
 * @author 	Ian Formanek
 */

public class XBorderLayout extends BorderLayout implements XelfiDesignLayout {
	private Hashtable components=new Hashtable();
	private boolean dirFree[] = new boolean[5];
	private boolean dirInit = false;

	
	public final static int XBL_NORTH=0;
	public final static int XBL_WEST=1;
	public final static int XBL_SOUTH=2;
	public final static int XBL_EAST=3;
	public final static int XBL_CENTER=4;

	private XelfiVisualContainer parent;

	private String getDirString(int dir) {
		switch (dir) {
			case XBL_NORTH: return "North";
			case XBL_WEST: return "West";
			case XBL_SOUTH: return "South";
			case XBL_EAST: return "East";
			case XBL_CENTER: return "Center";
			default: return null;
		}
	}
		
	public void setParent(XelfiVisualContainer aParent) {
		parent = aParent;
	}

	public int getPropertyCount() { return 1;}

	public String[] getPropertyNames() { 
		String list[] = new String[1];
		list[0] = new String("Direction");
		return list; 
	}

	public void addComponent(XelfiVisualComponent comp) {
		int dir=0;
		if (!dirInit) {
			dirInit = true;
			for (int i=0; i<5; i++)
				dirFree[i] = true;
			dir = 0;
		}
		else {
			for (int i=0; i<5; i++)
				if ((dirFree[i]) || (i==4)) {
					dir = i;
					break;
				}
		}
		dirFree[dir] = false;
		components.put(comp.getName(), new Integer(dir));
	}
	
	public void renameComponent(XelfiVisualComponent comp, String newName) throws 
		XFNameNotFoundException {
		String oldName = comp.getName();
		Object value = components.get(oldName);
		if (value == null)
			throw new XFNameNotFoundException();
		components.remove(oldName);
		components.put(newName, value);
	}
	
	public void removeComponent(XelfiVisualComponent comp) throws 
		XFNameNotFoundException {
		String compName = comp.getName();
		if (!components.containsKey(compName))
			throw new XFNameNotFoundException();
		components.remove(compName);
		setDirFree();
	}

	
	public int getPropertyViewerType(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		if (index == 0) {
			return XFDesignConstants.PVT_CHOICE_STRICT;
		}
		throw new XFPropertyIndexOutOfBoundsException();
	}

	public String[] getPropertyValueNames(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		if (index == 0) {
			String list[] = new String[5];
			for (int i= 0; i<5; i++) list[i] = getDirString(i);
			return list;
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}

	public Object[] getPropertyValueValues(int index) throws 
		XFPropertyIndexOutOfBoundsException {
		if (index == 0) {
			Integer list[] = new Integer[5];
			for (int i=0; i<5; i++) 
				list[i] = new Integer(i);
			return list;
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}

	public Object getProperty(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException {
		if (index != 0) throw new XFPropertyIndexOutOfBoundsException();
		Object value = components.get(comp.getName());
		if (value == null)
			throw new XFNameNotFoundException();
		return value;
	}
		
	public String getPropertyString(XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFNameNotFoundException {
		if (index != 0) throw new XFPropertyIndexOutOfBoundsException();
		Integer value = (Integer)components.get(comp.getName());
		if (value == null)
			throw new XFNameNotFoundException();
		String list[] = getPropertyValueNames(0);
		return list[value.intValue()];
	}
		
	private void setDirFree() {
		for (int i=0; i<5; i++)
			dirFree[i] = true;
		for (Enumeration e = components.elements() ; e.hasMoreElements() ;)
			dirFree[((Integer)e.nextElement()).intValue()] = false; 
	}		
	
	public int setProperty(XelfiVisualComponent comp, int index, Object value) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyBadValueTypeException,
		XFNameNotFoundException {
		if (index != 0) throw new XFPropertyIndexOutOfBoundsException();
		if (!(value instanceof Integer)) throw new XFPropertyBadValueTypeException();
		int ret = XFDesignConstants.RSP_OK;
		if (!components.containsKey(comp.getName()))
			throw new XFNameNotFoundException();
		components.put(comp.getName(), value);
		removeLayoutComponent((Component)comp);
		addLayoutComponent(getDirString(((Integer)value).intValue()),(Component)comp);
		((Component)comp).invalidate();
		((Component)comp).getParent().validate();
		setDirFree();
		comp.getParentForm().regenerateCode(comp, XFDesignConstants.GEN_ADD);
		return ret;
	}

	public void specialPropertyInput(Frame parent, XelfiVisualComponent comp, int index) throws 
		XFPropertyIndexOutOfBoundsException,
		XFPropertyNoSpecialInputException,
		XFNameNotFoundException {
		throw new XFPropertyNoSpecialInputException();
	}
		
	public String getClassName() {
		return "BorderLayout";
	}

	///////////////////
	// Layout things //
	///////////////////

	public boolean allowsDragging() { return false; }

	public void layoutAdd(Container cont, XelfiVisualComponent comp) {
		String compName = comp.getName();
		Integer dir = (Integer)components.get(compName);
		if (dir == null) {
			addComponent(comp);
			dir = (Integer)components.get(compName);
		}
		cont.add(getDirString(dir.intValue()), (Component)comp);
	}

	public void layoutAdd(Container cont, int pos, XelfiVisualComponent comp) {
		layoutAdd(cont, comp);  // [PENDING - how to add to certain position ???]
	}

	/////////////////////
	// Code generation //
	/////////////////////
	
	public String[] neededImports() {
		String list[] = new String[0];
		return list;
	}

	// this method returns whether it needs a variable for the layout
	public boolean genLayoutVariable() { return false; }
	
	public String genLayoutInitCode(String containerName, String varName) {
		if (containerName == null)
			return "setLayout(new BorderLayout());\n";
		else 
			return containerName+".setLayout(new BorderLayout());\n";
	}
	
	public String genComponentLayoutCode(String containerName, XelfiVisualComponent comp, String varName) {
		String compName = comp.getName();
		Integer direction = (Integer)components.get(compName);
		if (direction == null) return null;
		if (containerName == null)
			return "add(\""+getDirString(direction.intValue())+"\", "+compName+");";
		else 
			return containerName+".add(\""+getDirString(direction.intValue())+"\", "+compName+");";
	}
	/////////////////
	// Persistence //
	/////////////////

	public void loadFromStream(DataInput stream) throws
		IOException { 
		int count = stream.readInt();
		for (int i=0; i< count; i++) {	
			String name = stream.readUTF();
			int dir = stream.readInt();
			components.put(name, new Integer(dir));
		}
	}
	
	public void saveToStream(DataOutput stream) throws
		IOException { 
		stream.writeInt(components.size());
		for (Enumeration e = components.keys() ; e.hasMoreElements() ;) { 
			String key = (String)e.nextElement(); 
			stream.writeUTF(key); 
			stream.writeInt(((Integer)components.get(key)).intValue());
		}
	}
}	