package xelfi.design.inspector;

import java.util.Vector;
import java.awt.Event;

import xelfi.design.forms.XelfiDesignInspectable;

/**
 * XelfiComponentInspector interface is a main interface for Component 
 * Inspector window. CI window shows names and values of component properties 	
 * including events. Choice box shows components (one or more) currently selected
 * in "inspectable" window. "Inspectable" is a window which implements 
 * the XelfiDesignInspectable interface. Such a window can be viewed and 
 * edited by CI window then. There are two types of windows which will 
 * probably implement XelfiDesignInspectable interface in Xelfi system: 
 * Form window and Menu editor window.
 * 
 * version 1.31 - propertiesChanged method added
 * 
 * @author Dafe
 * @version 1.31, 26 Nov 1996
*/
public interface XelfiComponentInspector {

	/**
		Notification method. This method is called from inspectable window
		(for example Form window) when its selection changes.
		CI is responsible for asking attached inspectable about its new 
		selection. Inspectable window will be repainted with new content then.
	*/
	public void selectionChanged ();

	/**
	 * Notifies component inspector that values of properties have changed.
	 * Component inspector should somehow ask for new values and repaint.
	*/
	public void propertiesChanged ();

	/**
		* Second notification method. This method tells CI that inspectable is
		* being changed. CI will remember reference to the new inspectable. 
		* CI will use it when method "selectionChanged" is called.
	*/
	public void inspectableChanged (XelfiDesignInspectable inspectable);

	/**
	 * Returns currently attached inspectable source or null when no 
	 * inspectable is set (CI is empty)
	*/
	public XelfiDesignInspectable getInspectable ();

	/**
		* Returns a vector of components currently inspected in CI.
		* Returns null when no component is inspected now.
	*/
	public Vector getInspectedComponents ();

	/**
		* Responds to mouse or keyboard events in Component Inspector window.
		* Divides reaction functionality between several "reaction" methods - 
		* but there's no need to make them public.
	*/
	public boolean handleEvent (Event e);

	/**
	 * Shows specified page in CI (one of properties, events or layouts).
	 * Returns false when specified page doesn't exists for currently viewed 
	 * component or set of components. (some components will not have the layout
	 * page - for example). The page will remain active to the next setPage 
	 * call.
	*/
	public boolean setPage (int page);

	/**
	 * Returns currently selected page. Constants names describing available
	 * pages can be found in "constants class" for design part of Xelfi.
	 * (xelfi.design.util.XFDesignConstants).
	 * Returns -1 when no page (no component inspected)
	*/
	public int getPage ();

}


