/**
* Xelfi programming environment
*/

package xelfi.editor;
/**
* An acceptor for searching for text.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class TextSearch extends Object implements Acceptor {
  /** the text to search for */
  private String text;

  /** readed text */
  private StringBuffer buf = new StringBuffer ();

  /** case sensitive search? */
  private boolean sensitive;

  /** New searcher.
  * @param s the string to search for
  */
  public TextSearch (String s) {
    this (s, false);
  };

  /** New searcher.
  * @param s the string to search for
  * @param c case sensitive?
  */
  public TextSearch (String s, boolean c) {
    text = s;
    sensitive = c;
  };
  /**
  * should we continue or stop the action?
  * @param ch the character at specified position
  * @return true/false
  */
  public boolean accept (char ch) {
    buf.append (ch);

    int len = buf.length ();
    int l = text.length ();
    if (len >= l) {
      String s = buf.toString ().substring (len - l, len);
      if (len > l * 10) {
        buf = new StringBuffer (s);
      };
      return sensitive ? s.equals (text) : s.equalsIgnoreCase (text);
    } else {
      return false;
    }
  };

  /** Starts search at specified position.
  * @param p the start of search
  * @return the founded position
  * @exception EndOfTextException if end of text is reached
  */
  public Position find (Position p) throws EndOfTextException {
    p = p.find (this);
    for (int i = 1; i < text.length (); i++) {
      p = p.previousChar ();
    };
    return p;
  };

  /** Starts search at specified position.
  * @param p the start of search
  * @param d the direction, true = forward
  * @return the founded position
  * @exception EndOfTextException if end of text is reached
  */
  public Position find (Position p, boolean d) throws EndOfTextException {
    if (d) return find (p);
    else return p.find (this, false);
  };
};


