/** Executor class
* @author: Filip Dvorak
* @version: 1.2
*/


package xelfi.execution;

import java.io.*;

import xelfi.top.*;
import xelfi.util.*;
import xelfi.compiler.CompilerOptions;

public class Executor
{
    /** Run class with standart parameters.
      @param className class to run
      @return the new process
     */
  public static void execute(String className)
  {
    execute(className, RunOptions.getArguments());
  }


  private static Process doExec(String className, String args)
  {
    Process child = null;
    Runtime rt = Runtime.getRuntime();
    try
    {
      child = rt.exec(RunOptions.getInterpretPath() + " " + className + " " + args);
    }
    catch(Exception e)
    {
      new MessageBox("Error", "Failed to run the interpreter " + RunOptions.getInterpretPath() +
        " (" + e + ")").show();
      return null;
    }
    return child;
  }

    /** Run class with parameters.
      @param className class to run
      @param args arguments given to class
      @return the new process
     */
  public static void execute(String className, String args)
  {
    Process child = doExec(className, args);
    if(RunOptions.showIOWindow()) new SubprocessIOWindow(child, "Class " + className + " running.",
      "Finished (" + className + ").").show();
    else runStandart(child);
  }

    /** Run standart io copying threads for process.
      @param child the process
    */
  protected static void runStandart(Process child)
  {
    StreamPipe childIn, childOut, childErr;

    // childIn = new StreamPipe(child.getOutputStream(), System.in, false);
    childErr = new StreamPipe(System.err, child.getErrorStream(), false);
    childOut = new StreamPipe(System.out, child.getInputStream(), false);

    // Thread t1 =  new Thread(childIn);
    // t1.setPriority(Thread.MIN_PRIORITY);

    Thread t3 = new Thread(childErr);
    t3.setPriority(Thread.MIN_PRIORITY);

    Thread t2 = new Thread(childOut);
    t2.setPriority(Thread.MIN_PRIORITY);

    // t1.start();
    t3.start();
    t2.start();
  }

  public static void runApplet(String className)
  {
    String width, height;

    if(RunOptions.getAppletWidth() < 0 || RunOptions.getAppletHeight() < 0)
      height = width = "";
    else
    {
      width = new Integer(RunOptions.getAppletWidth()).toString();
      height = new Integer(RunOptions.getAppletHeight()).toString();
    }

    Process child = doExec("xelfi.execution.AppletViewer", className + " " + width + " " + height);
    if(RunOptions.showIOWindow()) new SubprocessIOWindow(child, "Applet " + className + " running.",
      "Finished (" + className + ").").show();
    else runStandart(child);
  }
}







































