/** "About ..." dialog box
* @author: Jan Formanek and Filip Dvorak
* @version: 1.2
*/

package xelfi.top.visual;

import java.awt.*;
import xelfi.top.TopDialog;
import xelfi.awt.ImageLabel;
import xelfi.util.PathScanner;
import xelfi.util.XelfiPath;
import java.io.File;

public class AboutBox extends TopDialog
{

  Button OKButton;
  public AboutBox(Frame parent)
  {
    super(parent, "About Xelfi IDE", true);
    setResizable(true);

    ColLayout aboxl = new ColLayout();
    setLayout(aboxl);
    ColLayoutProperties aboxp = new ColLayoutProperties();
      // xelfi logo image
    Panel logoPanel = new SizePanel(20, 15, 0.6f, 0.76f);
    add(logoPanel);
    aboxp.compInsets = new Insets(10, 10, 10, 10);
    aboxp.compGrowth = 0.92f;
    aboxl.add(logoPanel, aboxp);
          // rest panel
    Panel rp = new Panel();
    add(rp);
    aboxp.compInsets = new Insets(10, 0, 10, 10);
    aboxp.compGrowth = 0.08f;
    aboxl.add(rp, aboxp);

      // logo panel setup
    logoPanel.setLayout(new BorderLayout());
    String name = PathScanner.addFileSeparator(XelfiPath.get());
    name = name + "images" + File.separator + "Logo.gif";
    Image im = Toolkit.getDefaultToolkit().getImage(name);
    ImageLabel imageLabel = new ImageLabel(im);
    logoPanel.add("Center", imageLabel);

      // rest panel setup - text panel
    RowLayout rpl = new RowLayout();
    rp.setLayout(rpl);
    RowLayoutProperties rpp = new RowLayoutProperties();
    Panel txpanel = new Panel();
    rp.add(txpanel);
    rpp.topSpaceGrowth = 0.2f;
    rpp.compGrowth = 0.14f;
    rpl.add(txpanel, rpp);
      // unregistered text
    Label unreg = new Label("Unregistered version.", Label.CENTER);
    rp.add(unreg);
    rpp.compInsets = new Insets(8, 0, 0, 0);
    rpp.compGrowth = 0;
    rpp.topSpaceGrowth = 0.1f;
    rpl.add(unreg, rpp);
      // copyright text
    Label copyright = new Label("(c) 1996, 97 Xelfi Technologies", Label.CENTER);
    rp.add(copyright);
    rpp.compInsets = new Insets(0, 0, 0, 0);
    rpp.topSpaceGrowth = 0f;
    rpl.add(copyright, rpp);

      // OKButton
    OKButton = new Button("Close");
    rp.add(OKButton);
    rpp.topSpaceGrowth = 0.3f;
    rpp.compGrowth = 0.06f;
    rpp.compInsets = new Insets(15, 0, 4, 0);
    rpp.leftSpaceGrowth = rpp.rightSpaceGrowth = 0.36f;
    rpl.add(OKButton, rpp);

      // text panel setup
    ColLayout txpl = new ColLayout();
    txpanel.setLayout(txpl);
    ColLayoutProperties txpp = new ColLayoutProperties();
      // left text panel
    Panel ltxp = new Panel();
    txpanel.add(ltxp);
    txpp.rightSpaceGrowth = 1f;
    txpl.add(ltxp, txpp);
      // right text panel
    Panel rtxp = new Panel();
    txpanel.add(rtxp);
    txpp.rightSpaceGrowth = 0f;
    txpl.add(rtxp, txpp);

      // left text panel setup
    RowLayout ltxl = new RowLayout();
    RowLayoutProperties ltxpr = new RowLayoutProperties();
    ltxpr.topSpaceGrowth = 0.33f;
    ltxp.setLayout(ltxl);
    Label l = new Label("OS Architecture: ");
    ltxp.add(l);
    ltxl.add(l, ltxpr);
    l = new Label("OS Name: ");
    ltxp.add(l);
    ltxl.add(l, ltxpr);
    l = new Label("Java version: ");
    ltxp.add(l);
    ltxl.add(l, ltxpr);

      // right text panel setup
    RowLayout rtxl = new RowLayout();
    RowLayoutProperties rtxpr = new RowLayoutProperties();
    rtxpr.topSpaceGrowth = 0.33f;
    rtxp.setLayout(rtxl);
    l = new Label(System.getProperty("os.arch", "unknown"));
    rtxp.add(l);
    rtxl.add(l, rtxpr);
    l = new Label(System.getProperty("os.name", "unknown"));
    rtxp.add(l);
    rtxl.add(l, rtxpr);
    l = new Label(System.getProperty("java.version", "unknown"));
    rtxp.add(l);
    rtxl.add(l, rtxpr);

    pack();
  }

  public void show()
  {
    if(getPeer() == null) addNotify();
    Dimension scr = Toolkit.getDefaultToolkit().getScreenSize(), me = size();

    move((scr.width - me.width) / 2, (int)((scr.height - me.height) / 2.4));
    super.show();
  }

 public boolean action(Event e, Object arg)
  {
    if (e.target == OKButton)
    {
      hide();
      dispose();
      return true;
    }
    return false;
  }

  public boolean handleEvent(Event event)
  {
    if (event.id == Event.WINDOW_DESTROY)
    {
      hide();
      dispose();
      return true;
    }
    return super.handleEvent(event);
  }

  public Insets insets()   // Under JDK 1.0.2 for win95 insets doesnt work. This patch should help.
  {
    Insets i = super.insets();
    if(getPeer() != null && (i.top < 0 || i.bottom < 0 || i.left < 0 || i.right < 0))
      return new Insets(28, 4, 4, 4);
    else return i;
  }
}

  // This panel returns special values of preferred and minimum size
class SizePanel extends Panel
{
  private int minw, minh;
  float prefw, prefh;

  SizePanel(int width, int height, float prefwRatio, float prefhRatio)
  {
    super();
    minw = width;
    minh = height;
    prefw = prefwRatio;
    prefh = prefhRatio;
  }


  public Dimension minimumSize()
  {
    return new Dimension(minw, minh);
  }

  public Dimension preferredSize()
  {
    Dimension d = super.preferredSize();
    d.width *= prefw;
    d.height *= prefh;
    return d;
  }
}


