/**
* Xelfi
*/

package xelfi.util;

import java.util.Properties;

/* Class EnhProperties is a subclass of Properties (WARNING: The java Properties not Xelfi!).
 * It is a little bit more comfortable to use so as it supports int/float/boolean
 * property types and also setting properties.
 * (Yes, you guess right, it means Enhanced Properties)
 * @version 	1.00, 08 Oct 1996
 * @author 	Ian Formanek
 */

public class EnhProperties extends Properties {
	public EnhProperties() { super(); }
	public EnhProperties(Properties defaults) { super(defaults); }

	public int getIntProperty(String key, int defVal) {
		String value = getProperty(key);
		int ivalue;
		if (value == null) return defVal;
		try {
			ivalue = Integer.valueOf(value).intValue();
			} catch (NumberFormatException e) {
				ivalue = defVal;
			}
		return ivalue;
		}

	public float getFloatProperty(String key, float defVal) {
		String value = getProperty(key);
		float fvalue;
		if (value == null) return defVal;
		try {
			fvalue = Float.valueOf(value).floatValue();
			} catch (NumberFormatException e) {
				fvalue = defVal;
			}
		return fvalue;
		}

	public boolean getBooleanProperty(String key, boolean defVal) {
		String value = getProperty(key);
		if (value == null) return defVal;
		else return Boolean.valueOf(value).booleanValue();
		}

      // FILIP>>
	public java.awt.Rectangle getRectProperty(String key, java.awt.Rectangle defVal)
	{
      String val = getProperty(key);
      try
      {
        java.awt.Rectangle rect = new java.awt.Rectangle();
          // x
        int pos = val.indexOf(','), posnew;
        rect.x = Integer.parseInt(val.substring(0, pos));
          // y
        posnew = val.indexOf(',', ++pos);
        rect.y = Integer.parseInt(val.substring(pos, posnew));
        pos = posnew;
          // width
        posnew = val.indexOf('x', ++pos);
        rect.width = Integer.parseInt(val.substring(pos, posnew));
        pos = posnew;
          // height
        rect.height = Integer.parseInt(val.substring(++pos, val.length()));
        return rect;
      }
      catch(NullPointerException e) { return defVal; }
      catch(NumberFormatException e) { return defVal; }
      catch(StringIndexOutOfBoundsException e) { return defVal; }
    }

	public java.awt.Color getColorProperty(String key, java.awt.Color defVal)
	{
      String val = getProperty(key);
      try
      {
        int red, green, blue;
          // red
        int pos = val.indexOf(','), posnew;
        red = Integer.parseInt(val.substring(0, pos));
          // green
        posnew = val.indexOf(',', ++pos);
        green = Integer.parseInt(val.substring(pos, posnew));
        pos = posnew;
          // blue
        blue = Integer.parseInt(val.substring(++pos, val.length()));
        return new java.awt.Color(red, green, blue);
      }
      catch(NullPointerException e) { return defVal; }
      catch(NumberFormatException e) { return defVal; }
      catch(StringIndexOutOfBoundsException e) { return defVal; }
    }
      // <<FILIP

	// in addition here could be Long and Double (if necessary)

	public void setIntProperty(String key, int value) {
		put(key, Integer.toString(value));
		}

	public void setBooleanProperty(String key, boolean value) {
		if (value) put(key,"true");
		else put(key,"false");
		}

	public void setFloatProperty(String key, float value) {
		put(key,Float.toString(value));
		}

      // FILIP>>
	public void setRectProperty(String key, java.awt.Rectangle value)     {
      put(key, value.x + "," + value.y + "," + value.width + "x" + value.height);
    }

	public void setColorProperty(String key, java.awt.Color value)
    {
      put(key, value.getRed() + "," + value.getGreen() + "," + value.getBlue());
    }
      // <<FILIP
}