/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.awt.*;
import xelfi.editor.*;




// ROMAN 17.12.1996
import xelfi.top.TopLevel;
// ROMAN 17.12.1996

//FILIP
import xelfi.compiler.ErrorColorerEvidence;
//FILIP

/**
* This is class for standard Xelfi editor. Extends @see EditorFrameBase.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class JavaEditorFrame extends EditorFrameBase {

  /** the source for this editor */
  private Source source;

  /** the status line */
  private StatusLine status;

  /** Opens new editor for specified source.
  * @param s the source
  */
  public JavaEditorFrame (Source s) {
    super (s.getName (), s.getDocument ());
    source = s;

    // must be created after the source = s assignment
    add ("South", status = new StatusLine (this));
    pack ();

    //FILIP
    getStandardEditorCallback ().addBackgroundCallback (
      ErrorColorerEvidence.getReference (s.getName ())
    );
    //FILIP

    //HANZ
//    getStandardEditorCallback().changeSyntaxCallback(
//                                                     new xelfi.browser.document.MyEditorCallback());
    //HANZ

    // ROMAN 17.12.1996
    getStandardEditorCallback ().addBackgroundCallback (
      TopLevel.getDebugger ()
    );
      // musi byt zaruceno, ze Debugger uz je vykonstruovany
      // toto zatim zaruceno je
    // ROMAN 17.12.1996


    getStandardEditorCallback ().changeSyntaxCallback (new SyntaxComments (
      s.getDocument ()
    ));

    /*
    xelfi.design.forms.CallbackRequestEvent cre;
    cre = new xelfi.design.forms.CallbackRequestEvent(this, null);
    getSource ().getDocument ().handleEvent(null, cre);
    if (cre.arg != null)
       getStandardEditorCallback ().addBackgroundCallback ((BackgroundCallback)cre.arg);
    */

    Rectangle r = xelfi.top.TopLevel.getMenu ().bounds ();
    Point loc = location ();
    if (loc.y < r.y + r.height) {
      move (loc.x, r.y + r.height);
    };
  };


  /** Returns the source that is displayed in this window.
  * @return the source
  */
  public Source getSource () {
    return source;
  };

  /** Gets standard editor callback. It is used to specify
  * colors of characters.
  * To this callback new syntax and background callbacks
  * can be specified.
  * @return the standard editor callback. Or null if the standard editor callback
  * has been changed.
  */
  public StandardEditorCallback getStandardEditorCallback () {
    try {
      return (StandardEditorCallback)getTextEditor ().getEditorCallback ();
    } catch (Exception ex) {
      return null;
    }
  }

  /** Sets new text to status line. The text will disappear on next modification
  * of the document.
  * @param s the text to display in the line
  */
  public void setStatusLine (String s) {
    if (status != null) {
      status.setText (s);
    };
  };

  /** Closes references to the source.
  */
  public void dispose () {
    super.dispose ();
    status.close ();
    source = null;
    status = null;
  };
};


